<?php
/* compose.php – full-feature sender UI (June-2025) */

session_start();
if (empty($_SESSION['uid'])) { header('Location: login.php'); exit; }

require_once __DIR__ . '/db.php';
require_once __DIR__ . '/flash.php';

$uid = (int)$_SESSION['uid'];

/* ── one-time tables ─────────────────────────────────────── */
db()->exec("CREATE TABLE IF NOT EXISTS subject_pool(id INTEGER PRIMARY KEY,user_id INT,text TEXT)");
db()->exec("CREATE TABLE IF NOT EXISTS from_pool   (id INTEGER PRIMARY KEY,user_id INT,text TEXT)");

db()->exec("CREATE TABLE IF NOT EXISTS send_jobs (
  id INTEGER PRIMARY KEY AUTOINCREMENT,
  user_id      INT,
  template_ids TEXT,   subject_ids TEXT,  from_ids TEXT, smtp_ids TEXT,
  attach_path  TEXT,
  recipients   TEXT,   speed REAL,   total INT DEFAULT 0,
  sent INT DEFAULT 0,  opened INT DEFAULT 0,
  bounced INT DEFAULT 0, failed INT DEFAULT 0,
  status TEXT DEFAULT 'queued',                -- queued|running|stopped|done|error
  started DATETIME, finished DATETIME
)");

/* ── quota helper ────────────────────────────────────────── */
function quota_left(int $u): int {
    $cap = db()->query("SELECT plan_quota FROM users WHERE id=$u")->fetchColumn();
    $cap = (int)$cap;                                    // –1 = unlimited
    if ($cap < 0) return -1;

    $sent = db()->query("SELECT COALESCE(SUM(sent),0) FROM email_stats
                         WHERE user_id=$u AND day>=date('now','start of month')")->fetchColumn();
    return max(0, $cap - (int)$sent);
}

/* ── AJAX API ────────────────────────────────────────────── */
if ($_SERVER['REQUEST_METHOD']==='POST' && isset($_POST['action'])) {
  header('Content-Type: application/json');
  $a = $_POST['action'];

  /* 1 ▸ load pools */
  if ($a==='load_pools') {
      $tpl  = db()->prepare("SELECT id,name  FROM email_templates WHERE user_id=? ORDER BY id DESC");
      $sub  = db()->prepare("SELECT id,text  FROM subject_pool     WHERE user_id=? ORDER BY id DESC");
      $frm  = db()->prepare("SELECT id,text  FROM from_pool        WHERE user_id=? ORDER BY id DESC");
      $smtp = db()->prepare("SELECT id,host,port FROM smtp_servers WHERE user_id=? ORDER BY id DESC");
      foreach([$tpl,$sub,$frm,$smtp] as $st) $st->execute([$uid]);
      echo json_encode([
        'tpl'  => $tpl ->fetchAll(PDO::FETCH_ASSOC),
        'sub'  => $sub ->fetchAll(PDO::FETCH_ASSOC),
        'frm'  => $frm ->fetchAll(PDO::FETCH_ASSOC),
        'smtp' => $smtp->fetchAll(PDO::FETCH_ASSOC),
        'quota'=> quota_left($uid)
      ]);
      exit;
  }

  /* 2 ▸ add subjects / from-names */
  if ($a==='bulk_save') {
      $tbl = ($_POST['kind']==='sub') ? 'subject_pool' : 'from_pool';
      $ins = db()->prepare("INSERT INTO $tbl(user_id,text) VALUES(?,?)");
      $lines = array_filter(array_map('trim', explode("\n", $_POST['text']??'')));
      foreach($lines as $line) $ins->execute([$uid,substr($line,0,255)]);
      echo '{"ok":1}'; exit;
  }

  /* 3 ▸ delete rows */
  if ($a==='del_pool') {
      $tbl = ($_POST['kind']==='sub') ? 'subject_pool' : 'from_pool';
      $ids = array_filter(array_map('intval', explode(',', $_POST['ids']??'')));
      if($ids){
          $in = implode(',',array_fill(0,count($ids),'?'));
          db()->prepare("DELETE FROM $tbl WHERE user_id=? AND id IN ($in)")
             ->execute(array_merge([$uid],$ids));
      }
      echo '{"ok":1}'; exit;
  }

  /* 4 ▸ create job */
  if ($a==='create_job') {
      /* recipients -------------------------------------------------- */
      if (!empty($_FILES['recfile']['tmp_name'])) {
          $rec = 'uploads/rec_'.uniqid().'_'.basename($_FILES['recfile']['name']);
          move_uploaded_file($_FILES['recfile']['tmp_name'],$rec);
          $total = substr_count(file_get_contents($rec), "\n") + 1;
      } else {
          $list = array_filter(array_map('trim', explode("\n", $_POST['recipients']??'')));
          if (!$list) { echo '{"err":"no recipients"}'; exit; }
          if (count($list) > 100) { echo '{"err":"paste ≤100"}'; exit; }
          $rec='uploads/rec_'.uniqid().'.txt';
          file_put_contents($rec, implode("\n",$list));
          $total = count($list);
      }

      /* attachment -------------------------------------------------- */
      $att = ''; 
      if (!empty($_FILES['attach']['tmp_name'])) {
          if ($_FILES['attach']['size'] > 5*1024*1024) { echo '{"err":">5 MB"}'; exit; }
          $att = 'uploads/att_'.uniqid().'_'.basename($_FILES['attach']['name']);
          move_uploaded_file($_FILES['attach']['tmp_name'], $att);
      }

      /* quota ------------------------------------------------------- */
      $left = quota_left($uid);
      if ($left >= 0 && $total > $left) {
          echo json_encode(['err'=>"Quota exceeded ($left left)"]); exit;
      }

      /* insert job -------------------------------------------------- */
      db()->prepare("INSERT INTO send_jobs
            (user_id,template_ids,subject_ids,from_ids,smtp_ids,
             attach_path,recipients,speed,total)
            VALUES (?,?,?,?,?,?,?,?,?)")->execute([
              $uid,
              $_POST['tpl']  ?? '',
              $_POST['sub']  ?? '',
              $_POST['frm']  ?? '',
              $_POST['smtp'] ?? '',
              $att, $rec,
              max(0.1,min(14.0,(float)$_POST['speed'])),
              $total
      ]);
      $jobId = db()->lastInsertId();

      /* enqueue in Redis ------------------------------------------- */
      try {
          $r = new Redis();
          if (!$r->connect('127.0.0.1', 6380, 1.5)) throw new Exception('Cannot connect');
          $r->rPush('send_queue', $jobId);
      } catch (Throwable $e) {
          db()->prepare("UPDATE send_jobs SET status='error' WHERE id=?")->execute([$jobId]);
          echo json_encode(['err'=>'Redis offline – job not queued']); exit;
      }

      echo json_encode(['ok'=>1,'job'=>$jobId,'quota_left'=>$left<0?-1:$left-$total]);
      exit;
  }

  /* 5 ▸ live stats */
  if ($a==='stats') {
      $st = db()->prepare("SELECT status,sent,opened,bounced,failed,total
                           FROM send_jobs WHERE id=? AND user_id=?");
      $st->execute([(int)$_POST['job'],$uid]);
      $row = $st->fetch(PDO::FETCH_ASSOC);
      $row['quota_left'] = quota_left($uid);
      echo json_encode($row); exit;
  }

  /* 6 ▸ stop */
  if ($a==='stop'){
      db()->prepare("UPDATE send_jobs SET status='stop' WHERE id=? AND user_id=?")
         ->execute([(int)$_POST['job'],$uid]);
      echo '{"ok":1}'; exit;
  }

  echo '{"err":"bad action"}'; exit;
}
?>
<!doctype html><html lang="en"><head>
<meta charset="utf-8"><title>Compose &amp; Send</title>
<meta name="viewport" content="width=device-width,initial-scale=1">
<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/css/bootstrap.min.css" rel="stylesheet">
<link href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.3/font/bootstrap-icons.css">
<style>
body{background:#0d1117;color:#fff}
.badge-neon{background:#0ff;color:#000;font-weight:600;box-shadow:0 0 10px #0ff}
.btn-neon{position:relative;color:#0ff;border:1px solid #0ff;font-weight:600;text-shadow:0 0 6px #0ff}
.btn-neon::after{content:'';position:absolute;inset:0;border-radius:.35rem;box-shadow:0 0 12px #0ff;opacity:0;transition:.3s}
.btn-neon:hover::after{opacity:1}
</style></head><body class="d-flex flex-column min-vh-100">

<nav class="navbar navbar-dark bg-dark">
  <div class="container-fluid">
    <a class="navbar-brand fw-bold" href="dashboard.php">&larr;&nbsp;Dashboard</a>
    <a class="btn btn-outline-light btn-sm" href="logout.php">Logout</a>
  </div>
</nav>

<div class="container py-5 flex-grow-1">
  <?php flash_render(); ?>
  <h1 class="h4 mb-4">Compose &amp; Send</h1>

  <p class="mb-4">
    Remaining this month:
    <span id="quotaBadge" class="badge badge-neon">…</span>
  </p>

  <!-- TEMPLATES -->
  <div class="border rounded p-3 mb-4">
    <label class="form-label mb-1">Templates</label><br>
    <label><input type="checkbox" id="tplAll" onchange="chkAll('.tpl',this.checked)"> <strong>Select&nbsp;all</strong></label>
    <div id="tplBox" style="max-height:160px;overflow:auto"></div>
  </div>

  <div class="row g-4">
    <!-- SUBJECTS -->
    <div class="col-md-6">
      <label class="form-label">Subjects</label>
      <select id="sub" class="form-select" multiple size="6"></select>
      <textarea id="subNew" class="form-control form-control-sm mt-1" rows="2"
                placeholder="Add subjects – one per line"></textarea>
      <div class="mt-1">
        <button class="btn btn-sm btn-secondary" onclick="bulkSave('sub')">Save</button>
        <button class="btn btn-sm btn-danger"    onclick="delSel('sub')">Delete&nbsp;selected</button>
      </div>
    </div>
    <!-- FROM NAMES -->
    <div class="col-md-6">
      <label class="form-label">From names</label>
      <select id="frm" class="form-select" multiple size="6"></select>
      <textarea id="frmNew" class="form-control form-control-sm mt-1" rows="2"
                placeholder="Add From names – one per line"></textarea>
      <div class="mt-1">
        <button class="btn btn-sm btn-secondary" onclick="bulkSave('frm')">Save</button>
        <button class="btn btn-sm btn-danger"    onclick="delSel('frm')">Delete&nbsp;selected</button>
      </div>
    </div>
  </div>

  <!-- SMTP -->
  <div class="border rounded p-3 my-4">
    <label class="form-label mb-1">SMTP servers</label><br>
    <label><input type="checkbox" id="smtpAll" onchange="chkAll('.smtp',this.checked)"> <strong>Select&nbsp;all</strong></label>
    <div id="smtpBox" style="max-height:200px;overflow:auto"></div>
  </div>

  <!-- RECIPIENTS -->
  <div class="mb-3">
    <label class="form-label">Recipients (paste ≤100)</label>
    <textarea id="rcpt" class="form-control" rows="6"></textarea>
  </div>
  <div class="mb-3">
    <label class="form-label">…or upload larger <code>.txt</code></label>
    <input type="file" id="recFile" class="form-control" accept=".txt">
  </div>

  <!-- ATTACHMENT -->
  <div class="mb-3">
    <label class="form-label">Attachment (optional, ≤5 MB)</label>
    <input type="file" id="attach" class="form-control">
  </div>

  <!-- SPEED SLIDER -->
  <div class="mb-4">
    <label class="form-label">
      Speed: <span id="spdVal">5.0</span>&nbsp;mails&nbsp;/&nbsp;s
      <small class="text-white-50 ms-2">(0.1 = 1 mail / 10 s)</small>
    </label>
    <input type="range" id="speed" class="form-range"
           min="0.1" max="14" step="0.1" value="5"
           oninput="spdVal.innerText=this.value">
  </div>

  <button class="btn btn-neon px-5" id="launch">Launch&nbsp;job</button>

  <!-- LIVE STATS -->
  <div id="statBox" class="alert alert-info mt-4 d-none">
    <button class="btn-close float-end" onclick="stopJob()"></button>
    <div id="statMsg">Queued…</div>
  </div>
</div>

<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js"></script>
<script>
/*-------- helper shortcuts --------*/
const api = fd => fetch('compose.php',{method:'POST',body:fd}).then(r=>r.json());
const $   = sel => document.querySelector(sel);
function chkAll(q,st){document.querySelectorAll(q).forEach(c=>c.checked=st);}
function opt(sel){return [...$(sel).selectedOptions].map(o=>o.value);}
function toast(msg,cls='danger'){alert(msg);}   // quick popup

/*-------- quota badge -------------*/
let quotaLeft=-1;
function badge(v){quotaLeft=v;quotaBadge.textContent=v<0?'Unlimited':v;}

/*-------- initial load ------------*/
async function load(){
  const fd=new FormData();fd.append('action','load_pools');
  const d=await api(fd);
  badge(d.quota);

  tplBox.innerHTML='';
  d.tpl.forEach(t=>tplBox.insertAdjacentHTML('beforeend',
    `<label class="d-block"><input type="checkbox" class="tpl" value="${t.id}"> ${t.name}</label>`));

  fill($('#sub'),d.sub,'text');
  fill($('#frm'),d.frm,'text');

  smtpBox.innerHTML='';
  d.smtp.forEach(s=>smtpBox.insertAdjacentHTML('beforeend',
    `<label class="d-block"><input type="checkbox" class="smtp" value="${s.id}">
      ${s.host}:${s.port}</label>`));
}
function fill(sel,arr,k){sel.innerHTML='';arr.forEach(x=>sel.insertAdjacentHTML('beforeend',
  `<option value="${x.id}">${x[k]}</option>`));}

/*-------- pool ops ----------------*/
function bulkSave(k){
  const ta=k==='sub'?$('#subNew'):$('#frmNew');
  if(!ta.value.trim())return;
  const fd=new FormData();fd.append('action','bulk_save');fd.append('kind',k);fd.append('text',ta.value);
  api(fd).then(()=>{ta.value='';load();});
}
function delSel(k){
  const sel=k==='sub'?$('#sub'):$('#frm');
  const ids=[...sel.selectedOptions].map(o=>o.value);
  if(!ids.length||!confirm(`Delete ${ids.length}?`))return;
  const fd=new FormData();fd.append('action','del_pool');fd.append('kind',k);fd.append('ids',ids.join(','));
  api(fd).then(()=>load());
}

/*-------- launch ------------------*/
let jobId=null;
$('#launch').onclick=async ()=>{
  if(!$('.tpl:checked')) return toast('Pick at least one template');
  if(!$('.smtp:checked'))return toast('Pick at least one SMTP');

  const fd=new FormData();fd.append('action','create_job');
  fd.append('tpl', [...document.querySelectorAll('.tpl:checked')].map(c=>c.value).join(','));
  fd.append('sub', opt('#sub').join(','));
  fd.append('frm', opt('#frm').join(','));
  fd.append('smtp',[...document.querySelectorAll('.smtp:checked')].map(c=>c.value).join(','));
  fd.append('recipients',$('#rcpt').value);
  fd.append('speed',$('#speed').value);
  if($('#recFile').files[0]) fd.append('recfile',$('#recFile').files[0]);
  if($('#attach').files[0])  fd.append('attach' ,$('#attach').files[0]);

  const r=await api(fd);
  if(!r.ok){toast(r.err||'Error');return;}
  badge(r.quota_left);
  jobId=r.job;
  $('#statBox').classList.remove('d-none');
  $('#statMsg').textContent='Job '+jobId+' queued…';
  poll();
};

/*-------- live stats --------------*/
async function poll(){
  if(!jobId)return;
  const fd=new FormData();fd.append('action','stats');fd.append('job',jobId);
  const s=await api(fd);
  if(s){
    badge(s.quota_left);
    $('#statMsg').innerHTML=`<strong style="text-transform:capitalize">${s.status}</strong>
      – ${s.sent}/${s.total} sent&nbsp;|&nbsp;${s.bounced+s.failed} failed`;
    if(s.status==='running'||s.status==='queued') setTimeout(poll,2000);
  }
}

/*-------- stop --------------------*/
function stopJob(){
  if(!confirm('Stop current job?'))return;
  const fd=new FormData();fd.append('action','stop');fd.append('job',jobId);
  api(fd).then(()=>$('#statMsg').textContent='Stopping…');
}

load();
</script>
</body></html>
