#!/usr/bin/env python3
# send_worker.py  –  SMTP sender for Mailer Nebula  (June-2025)

import os, ssl, smtplib, time, datetime, random, itertools, sqlite3, redis
from email.mime.multipart import MIMEMultipart
from email.mime.text      import MIMEText
from email.mime.base      import MIMEBase
from email                import encoders

# ── config ──────────────────────────────────────────────────────────────
ROOT          = os.path.dirname(__file__)
DB_PATH       = os.path.join(ROOT, 'mailer.db')
REDIS_HOST    = '127.0.0.1'
REDIS_PORT    = 6380          # ← your Docker Redis
QUEUE_NAME    = 'send_queue'

HEALTH_WINDOW = 3             # consecutive fails → mark “bad”
STATS_FLUSH_EVERY = 8         # seconds between DB flushes
MAX_RCPT_ERRS = 50            # safety-valve: abort job if >50 hard fails

# ── helpers ─────────────────────────────────────────────────────────────
def db():
    cx = sqlite3.connect(DB_PATH, isolation_level=None,
                         detect_types=sqlite3.PARSE_DECLTYPES)
    cx.row_factory = sqlite3.Row
    return cx

def pixelize(html: str, job_id: int, rcpt: str) -> str:
    px = (f'<img src="https://yourdomain.com/pixel.php?id={job_id}&r={rcpt}" '
          f'width="1" height="1" style="display:none" alt="">')
    return html.replace('</body>', px + '</body>')

def rr_cycle(lst):
    """round-robin generator (len ≥ 1)"""
    return itertools.cycle(lst)

def today():
    return datetime.date.today().isoformat()

# ── SMTP wrapper ────────────────────────────────────────────────────────
class SMTPConn:
    def __init__(self, row: sqlite3.Row):
        self.row = row
        self.con = None
        self.fails = 0

    def connect(self):
        host, port = self.row['host'], int(self.row['port'])
        ctx = ssl.create_default_context()
        if port == 465:
            self.con = smtplib.SMTP_SSL(host, port, timeout=12, context=ctx)
        else:
            self.con = smtplib.SMTP(host, port, timeout=12)
            self.con.ehlo()
            if port in (587, 2525):
                self.con.starttls(context=ctx)
        if self.row['username']:
            self.con.login(self.row['username'], self.row['password'])

    def send(self, msg, rcpt):
        try:
            if self.con is None:
                self.connect()
            self.con.sendmail(msg['From'], [rcpt], msg.as_string())
            self.fails = 0
            return True
        except Exception:
            self.fails += 1
            try:
                if self.con:
                    self.con.quit()
            except Exception:
                pass
            self.con = None
            return False

# ── main worker loop ───────────────────────────────────────────────────
def main():
    rds = redis.Redis(host=REDIS_HOST, port=REDIS_PORT, decode_responses=True)
    sql = db()

    print('Worker started – waiting for jobs …')
    while True:
        item = rds.blpop(QUEUE_NAME, timeout=5)
        if not item:
            continue
        job_id = int(item[1])
        print('[*] Picked job', job_id)

        job = sql.execute("SELECT * FROM send_jobs WHERE id=?", (job_id,)).fetchone()
        if not job:
            continue

        sql.execute("UPDATE send_jobs SET status='running',started=CURRENT_TIMESTAMP WHERE id=?", (job_id,))

        # --- pools ---------------------------------------------------
        def rows(table, idcsv):
            if not idcsv: return []
            ids = [int(x) for x in idcsv.split(',') if x]
            q = f"SELECT * FROM {table} WHERE id IN ({','.join(['?']*len(ids))})"
            return list(sql.execute(q, ids))

        tpl   = rows('email_templates', job['template_ids'])
        subs  = rows('subject_pool',    job['subject_ids'])
        frms  = rows('from_pool',       job['from_ids'])
        smtps = rows('smtp_servers',    job['smtp_ids'])

        if not tpl or not smtps:
            sql.execute("UPDATE send_jobs SET status='error' WHERE id=?", (job_id,))
            print('[!] Missing pools – job', job_id, 'aborted')
            continue

        tpl_c  = rr_cycle(tpl)
        sub_c  = rr_cycle(subs or [{'text': 'Hello'}])
        frm_c  = rr_cycle(frms or [{'text': 'Mailer'}])
        smtp_c = rr_cycle([SMTPConn(row) for row in smtps])

        # --- recipients ---------------------------------------------
        with open(job['recipients'], encoding='utf-8', errors='ignore') as f:
            rcpts = [l.strip() for l in f if l.strip()]

        speed = max(0.1, min(14.0, job['speed']))
        delay = 1.0 / speed

        attach = job['attach_path'] if job['attach_path'] and os.path.isfile(job['attach_path']) else None

        sent = opened = bounced = failed = 0
        last = time.time()
        hard_fails = 0

        for idx, rcpt in enumerate(rcpts, 1):
            # graceful stop
            if sql.execute("SELECT status FROM send_jobs WHERE id=?", (job_id,)).fetchone()['status'] == 'stop':
                sql.execute("UPDATE send_jobs SET status='stopped' WHERE id=?", (job_id,))
                print('[!] Job', job_id, 'stopped by user')
                break

            smtp = next(smtp_c)
            msg = MIMEMultipart()
            msg['Subject'] = next(sub_c)['text']
            frm = next(frm_c)['text']
            msg['From'] = f'{frm} <{smtp.row["from_email"]}>'
            msg['To']   = rcpt

            body = pixelize(next(tpl_c)['html'], job_id, rcpt)
            msg.attach(MIMEText(body, 'html', 'utf-8'))

            if attach:
                with open(attach, 'rb') as af:
                    part = MIMEBase('application', 'octet-stream')
                    part.set_payload(af.read())
                    encoders.encode_base64(part)
                    part.add_header('Content-Disposition',
                                    f'attachment; filename="{os.path.basename(attach)}"')
                    msg.attach(part)

            ok = smtp.send(msg, rcpt)
            if ok:
                sent += 1
            else:
                failed += 1
                hard_fails += 1
                if smtp.fails >= HEALTH_WINDOW:
                    sql.execute("UPDATE smtp_servers SET last_health='bad' WHERE id=?", (smtp.row['id'],))

            # periodic DB flush
            if time.time() - last >= STATS_FLUSH_EVERY or idx == len(rcpts):
                sql.execute("""UPDATE send_jobs
                               SET sent = sent + ?,
                                   failed = failed + ?
                               WHERE id=?""",
                            (sent, failed, job_id))
                day = today()
                sql.execute("""INSERT INTO email_stats(user_id, day, sent)
                               VALUES (?,?,?)
                               ON CONFLICT(user_id,day) DO UPDATE
                               SET sent = sent + excluded.sent""",
                            (job['user_id'], day, sent))
                sent = failed = 0
                last = time.time()

            if hard_fails >= MAX_RCPT_ERRS:
                sql.execute("UPDATE send_jobs SET status='error' WHERE id=?", (job_id,))
                print('[!] Too many failures – job', job_id, 'aborted')
                break

            time.sleep(delay)

        # mark done if not errored / stopped
        st = sql.execute("SELECT status FROM send_jobs WHERE id=?", (job_id,)).fetchone()['status']
        if st == 'running':
            sql.execute("UPDATE send_jobs SET status='done',finished=CURRENT_TIMESTAMP WHERE id=?", (job_id,))
            print('[✓] Job', job_id, 'finished')

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print('^C – worker stopped')
