#!/usr/bin/env python3
"""
Usage: python smtptest.py host port user pass from_email recipient_csv

• Sends a plain-text test mail to every comma-separated recipient.
• SUCCESS (“✅ Mail accepted for all recipients”) only when:
    – TCP/TLS connection OK
    – (optional) AUTH OK
    – SMTP server returns 250 after DATA
    – NO recipient was refused (sendmail() returns an empty dict)
• Prints details of any refusal or exception.
• Returns exit-code 0 on success, 1 on any failure.
"""

import sys, smtplib, ssl, datetime, random, string

if len(sys.argv) != 7:
    print("❌ args: host port user pass from_email recipients", file=sys.stderr)
    sys.exit(1)

host, port, user, pwd, from_addr, recip_csv = sys.argv[1:]
port = int(port)
recipients = [r.strip() for r in recip_csv.split(",") if r.strip()]

# ── unique subject/body bits ──────────────────────────────
tag     = ''.join(random.choices(string.ascii_uppercase + string.digits, k=4))
subject = f"SMTP-Test {tag}"
body_id = ''.join(random.choices(string.ascii_letters + string.digits, k=8))

msg = f"""From: {from_addr}
To: {", ".join(recipients)}
Subject: {subject}
Date: {datetime.datetime.utcnow():%a, %d %b %Y %H:%M:%S +0000}

Automated test from Mailer Nebula

SMTP server : {host}:{port}
SMTP user   : {user or "(no login)"}
Unique body : {body_id}
"""

def connect(use_ssl: bool):
    cls = smtplib.SMTP_SSL if use_ssl else smtplib.SMTP
    context = ssl.create_default_context()
    return cls(host, port, timeout=10, context=context) if use_ssl else cls(host, port, timeout=10)

def send(use_ssl: bool):
    with connect(use_ssl) as s:
        s.ehlo()
        if not use_ssl and port in (587, 2525):
            s.starttls(context=ssl.create_default_context())
            s.ehlo()
        if user:
            s.login(user, pwd)
        refused = s.sendmail(from_addr, recipients, msg.encode())
        return refused   # dict {recipient: (code, resp)}

try:
    refused = {}
    if port == 465:
        refused = send(True)
    else:
        try:
            refused = send(False)
        except smtplib.SMTPNotSupportedError:
            refused = send(True)        # implicit SSL fallback

    if refused:
        for rcpt, (code, resp) in refused.items():
            print(f"❌  {rcpt} refused ({code} {resp.decode() if isinstance(resp, bytes) else resp})",
                  file=sys.stderr)
        sys.exit(1)

    print("✅  Mail accepted for all recipients")
    sys.exit(0)

except Exception as e:
    print(f"❌  {e}", file=sys.stderr)
    sys.exit(1)
