<?php
/* wallet.php – NOWPayments wallet (plans-aware, May 2025) */
session_start();
if (empty($_SESSION['uid'])) { header('Location: login.php'); exit; }

require_once __DIR__.'/config.php';
require_once __DIR__.'/db.php';

$uid = (int)$_SESSION['uid'];

/* ── DB bootstrap / migration ─────────────────────────── */
db()->exec("CREATE TABLE IF NOT EXISTS wallet_tx (
  id INTEGER PRIMARY KEY AUTOINCREMENT,
  user_id      INTEGER,
  amount_usd   REAL,
  crypto       TEXT,
  payment_id   TEXT,
  pay_address  TEXT,
  invoice_url  TEXT,
  status       TEXT DEFAULT 'pending',   -- pending|confirmed|failed|spent
  created_at   DATETIME DEFAULT CURRENT_TIMESTAMP
)");
$cols = db()->query("PRAGMA table_info(wallet_tx)")->fetchAll(PDO::FETCH_COLUMN,1);
if (!in_array('invoice_url', $cols)) {
  db()->exec("ALTER TABLE wallet_tx ADD COLUMN invoice_url TEXT");
}

/* ── NOWPayments helper ───────────────────────────────── */
function nowpay(string $method,string $ep,array $pl=null):array{
  $ch=curl_init("https://api.nowpayments.io/v1/$ep");
  curl_setopt_array($ch,[CURLOPT_RETURNTRANSFER=>1,CURLOPT_TIMEOUT=>25,
    /* --- DEV ONLY --- */CURLOPT_SSL_VERIFYPEER=>false,CURLOPT_SSL_VERIFYHOST=>false,
    CURLOPT_HTTPHEADER=>['x-api-key: '.NOWPAY_API_KEY,'Content-Type: application/json'],
    CURLOPT_CUSTOMREQUEST=>$method]);
  if($pl) curl_setopt($ch,CURLOPT_POSTFIELDS,json_encode($pl,JSON_UNESCAPED_SLASHES));
  $json=curl_exec($ch);
  if($json===false) throw new Exception('cURL: '.curl_error($ch));
  $code=curl_getinfo($ch,CURLINFO_HTTP_CODE); $data=json_decode($json,true)??[];
  if($code>=300) throw new Exception('NOWPayments: '.($data['message']??"HTTP $code"));
  return $data;
}
/* ── send credit e-mail (optional) ─────────────────────── */
function mail_credit(int $uid,float $amt):void{
  $sto=db()->prepare("SELECT email FROM users WHERE id=?");$sto->execute([$uid]);
  if($to=$sto->fetchColumn()){
    @mail($to,"Wallet credited \$$amt",
      "Hi,\nYour wallet was credited with \$$amt USD.\n",
      "From: ".SITE_MAIL_FROM);
  }
}

/* ── JSON API ─────────────────────────────────────────── */
if ($_SERVER['REQUEST_METHOD']==='POST' && isset($_POST['action'])){
  header('Content-Type: application/json'); $a=$_POST['action'];

  /* list + true balance (confirmed + spent) */
  if($a==='list'){
    $rows=db()->prepare("SELECT * FROM wallet_tx WHERE user_id=? ORDER BY id DESC");
    $rows->execute([$uid]);
    $bal=db()->prepare("
      SELECT COALESCE(SUM(CASE
               WHEN status IN ('confirmed','spent') THEN amount_usd
               ELSE 0 END),0)
      FROM wallet_tx WHERE user_id=?");
    $bal->execute([$uid]);
    echo json_encode(['bal'=>$bal->fetchColumn(),'rows'=>$rows->fetchAll(PDO::FETCH_ASSOC)]); exit;
  }

  /* ------- (other actions unchanged) ------- */
  /* … everything from ‘create_inv’, ‘poll’, ‘delete_bulk’, ‘delete_all’
       remains exactly the same as in your previous working version … */

  /* create invoice */
  if($a==='create_inv'){
    $amt=max(10,(float)$_POST['amt']); $crypto=in_array($_POST['crypto'],['BTC','ETH','LTC'])?$_POST['crypto']:'BTC';
    $cnt=db()->prepare("SELECT COUNT(*) FROM wallet_tx WHERE user_id=? AND date(created_at)=date('now')");$cnt->execute([$uid]);
    if($cnt->fetchColumn()>=5){echo json_encode(['err'=>'Daily limit 5']);exit;}
    try{
      $inv=nowpay('POST','invoice',[
        'price_amount'=>$amt,'price_currency'=>'usd','pay_currency'=>strtolower($crypto),
        'order_id'=>'UID'.$uid.'-'.time(),'ipn_callback_url'=>'https://yourdomain.com/mailer/ipn.php'
      ]);
    }catch(Exception $e){echo json_encode(['err'=>$e->getMessage()]);exit;}
    db()->prepare("INSERT INTO wallet_tx
      (user_id,amount_usd,crypto,payment_id,pay_address,invoice_url)
      VALUES (?,?,?,?,?,?)")->execute([$uid,$amt,$crypto,$inv['payment_id']??'',
                                       $inv['pay_address']??'',$inv['invoice_url']??'']);
    echo json_encode(['ok'=>1,'link'=>$inv['invoice_url']??'#']);exit;
  }

  /* poll (unchanged except status list covers spent rows automatically) */
  if($a==='poll'){
    $pend=db()->prepare("SELECT * FROM wallet_tx WHERE user_id=? AND status='pending'");$pend->execute([$uid]);
    foreach($pend as $row){
      try{
        $p=nowpay('GET','payment/'.$row['payment_id']); $s=strtolower($p['payment_status']??'');
        if($s==='confirmed'){
          db()->prepare("UPDATE wallet_tx SET status='confirmed' WHERE id=?")->execute([$row['id']]);
          mail_credit($uid,$row['amount_usd']);
        }elseif($s==='failed'){
          db()->prepare("UPDATE wallet_tx SET status='failed' WHERE id=?")->execute([$row['id']]);
        }
      }catch(Exception $e){}
    }
    db()->prepare("UPDATE wallet_tx SET status='failed'
      WHERE user_id=? AND status='pending' AND datetime(created_at,'+1 day')<datetime('now')")
      ->execute([$uid]);
    echo '{"ok":1}';exit;
  }

  /* delete selected */
  if($a==='delete_bulk'){
    $ids=array_filter(array_map('intval',explode(',',$_POST['ids']??'')));
    if(!$ids){echo '{"err":"no ids"}';exit;}
    $in=implode(',',array_fill(0,count($ids),'?'));
    db()->prepare("DELETE FROM wallet_tx WHERE user_id=? AND id IN ($in)")
       ->execute(array_merge([$uid],$ids));
    echo '{"ok":1}';exit;
  }
  /* delete all */
  if($a==='delete_all'){
    db()->prepare("DELETE FROM wallet_tx WHERE user_id=?")->execute([$uid]);echo '{"ok":1}';exit;
  }

  echo '{"err":"bad action"}';exit;
}
?>
<!doctype html><html lang="en"><head>
<meta charset="utf-8"><title>Wallet</title>
<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/css/bootstrap.min.css" rel="stylesheet">
<link href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.3/font/bootstrap-icons.css" rel="stylesheet">
<style>
body{background:#0d1117;color:#fff}
.btn-neon{position:relative;color:#0ff;border:1px solid #0ff;text-shadow:0 0 6px #0ff}
.btn-neon::after{content:'';position:absolute;inset:0;box-shadow:0 0 12px #0ff;opacity:0;transition:.3s}
.btn-neon:hover::after{opacity:1}
</style></head><body class="d-flex flex-column min-vh-100">

<nav class="navbar navbar-dark bg-dark">
  <div class="container-fluid">
    <a class="navbar-brand" href="dashboard.php">&larr; Dashboard</a>
    <a class="btn btn-outline-light btn-sm" href="logout.php">Logout</a>
  </div>
</nav>

<main class="container py-5 flex-grow-1">
  <h1 class="h4 mb-3">Wallet</h1>

  <div class="card text-dark bg-info mb-4" style="max-width:18rem">
    <div class="card-body"><h5 class="card-title">Balance</h5>
      <p class="h3" id="bal">$0.00</p>
    </div>
  </div>

  <div class="mb-3">
    <button class="btn btn-neon btn-sm me-2" data-bs-toggle="modal" data-bs-target="#addModal">
      <i class="bi bi-plus-circle"></i> Add funds
    </button>
    <button id="btnPoll" class="btn btn-secondary btn-sm me-2">
      <i class="bi bi-arrow-repeat"></i> Refresh status
    </button>
    <button id="btnDelSel" class="btn btn-danger btn-sm me-2" disabled>
      <i class="bi bi-trash"></i> Delete selected
    </button>
    <button id="btnDelAll" class="btn btn-danger btn-sm">
      <i class="bi bi-trash-fill"></i> Delete ALL history
    </button>
  </div>

  <div class="table-responsive">
    <table class="table table-dark table-sm text-center align-middle" id="tbl">
      <thead><tr>
        <th><input type="checkbox" id="selAll"></th>
        <th>ID</th><th>Date</th><th>USD</th><th>Crypto</th><th>Status</th><th></th>
      </tr></thead>
      <tbody></tbody>
    </table>
  </div>
</main>

<!-- Add funds modal (unchanged) -->
<div class="modal fade" id="addModal" tabindex="-1"><div class="modal-dialog">
<div class="modal-content bg-dark text-white">
  <div class="modal-header"><h5 class="modal-title">Add funds</h5>
    <button class="btn-close btn-close-white" data-bs-dismiss="modal"></button>
  </div>
  <div class="modal-body">
    <form id="addForm">
      <div class="mb-3">
        <label class="form-label">Amount (USD) <small>min $10</small></label>
        <input type="number" name="amt" min="10" step="0.01" class="form-control" required>
      </div>
      <div class="mb-3">
        <label class="form-label">Crypto</label>
        <select name="crypto" class="form-select"><option>BTC</option><option>ETH</option><option>LTC</option></select>
      </div>
      <button class="btn btn-neon">Generate invoice</button>
    </form>
    <div id="invBox" class="mt-3 d-none">
      <p class="small text-white-50">Pay with the secure NOWPayments link:</p>
      <a id="invLink" class="link-info" target="_blank"></a>
    </div>
  </div>
</div></div></div>

<footer class="text-center text-white-50 py-3 small">&copy; <?=date('Y')?> Mailer Nebula</footer>

<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js"></script>
<script>
const api=fd=>fetch('wallet.php',{method:'POST',body:fd}).then(r=>r.json());

function toast(t,c='info'){const d=document.createElement('div');
d.className=`alert alert-${c} position-fixed top-0 end-0 m-3 py-2 px-3 fade`;d.style.zIndex=1080;
d.textContent=t;document.body.append(d);new bootstrap.Alert(d);
setTimeout(()=>d.classList.add('show'),10);setTimeout(()=>d.classList.remove('show'),3000);}

function cls(s){return{confirmed:'success',pending:'warning',failed:'danger',spent:'secondary'}[s]||'secondary';}

function load(){
  const fd=new FormData();fd.append('action','list');
  api(fd).then(r=>{
    bal.textContent='$'+parseFloat(r.bal).toFixed(2);
    const tb=document.querySelector('#tbl tbody');tb.innerHTML='';
    r.rows.forEach(x=>{
      tb.insertAdjacentHTML('beforeend',`
<tr data-id="${x.id}">
  <td><input type="checkbox" class="sel"></td>
  <td>${x.id}</td>
  <td>${x.created_at.split(' ')[0]}</td>
  <td>$${x.amount_usd.toFixed(2)}</td>
  <td>${x.crypto}</td>
  <td><span class="badge bg-${cls(x.status)}">${x.status}</span></td>
  <td>${x.invoice_url?`<a class="link-info" href="${x.invoice_url}" target="_blank">invoice</a>`:''}</td>
</tr>`);});
    syncSel();
  });
}

/* selection helpers */
selAll.onchange=()=>document.querySelectorAll('.sel').forEach(c=>c.checked=selAll.checked);
document.addEventListener('change',e=>{if(e.target.classList.contains('sel'))syncSel();});
function selected(){return[...document.querySelectorAll('.sel:checked')].map(c=>c.closest('tr').dataset.id);}
function syncSel(){btnDelSel.disabled=!selected().length; selAll.checked=[...document.querySelectorAll('.sel')].every(c=>c.checked);}

/* create invoice (unchanged) */
addForm.addEventListener('submit',async e=>{
  e.preventDefault();invBox.classList.add('d-none');
  const b=e.submitter,orig=b.innerHTML;b.disabled=true;
  b.innerHTML='<span class="spinner-border spinner-border-sm me-1"></span>Generating…';
  const fd=new FormData(e.target);fd.append('action','create_inv');
  const r=await api(fd);b.disabled=false;b.innerHTML=orig;
  if(r.err){toast(r.err,'danger');return;}
  invLink.href=r.link;invLink.textContent=r.link;invBox.classList.remove('d-none');
  toast('Invoice generated','success');load();
});

/* poll / delete (unchanged) */
btnPoll.onclick=()=>{const fd=new FormData();fd.append('action','poll');api(fd).then(()=>{toast('Refreshed','info');load();});};
btnDelSel.onclick=()=>{const ids=selected();if(!ids.length)return;if(confirm(`Delete ${ids.length} tx?`)){const fd=new FormData();fd.append('action','delete_bulk');fd.append('ids',ids.join(','));api(fd).then(()=>{toast('Deleted','warning');load();});}};
btnDelAll.onclick=()=>{if(confirm('Delete ALL transactions?')){const fd=new FormData();fd.append('action','delete_all');api(fd).then(()=>{toast('All cleared','warning');load();});}};

/* init */
load();
</script>
</body></html>
